@@ -10,11 +10,13 @@ Crypt::Perl::RSA::Parse - RSA key parsing
 
     use Crypt::Perl::RSA::Parse ();
 
-    #These accept either DER or PEM, native format or PKCS8.
-    #
+    # These accept either DER or PEM, native format or PKCS8/SPKI.
     my $prkey = Crypt::Perl::RSA::Parse::private($buffer);
     my $pbkey = Crypt::Perl::RSA::Parse::public($buffer);
 
+    # Note that this accepts a structure, not raw JSON.
+    my $key = Crypt::Perl::RSA::Parse::jwk($jwk_hr);
+
 =head1 DISCUSSION
 
 See L<Crypt::Perl::RSA::PrivateKey> and L<Crypt::Perl::RSA::PublicKey>
@@ -28,10 +30,10 @@ use warnings;
 use Try::Tiny;
 
 use Crypt::Format ();
-use Module::Load ();
 
 use Crypt::Perl::ASN1 ();
 use Crypt::Perl::RSA::Template ();
+use Crypt::Perl::X ();
 
 sub _asn1 {
     return Crypt::Perl::ASN1->new()->prepare(
@@ -57,7 +59,7 @@ sub private {
             $key_obj = private_pkcs8($pem_or_der);
         }
         catch {
-            die "Failed to parse as either RSA ($rsa_err) or PKCS8 ($_)";
+            die Crypt::Perl::X::create('Generic', "Failed to parse as either RSA ($rsa_err) or PKCS8 ($_)");
         };
     };
 
@@ -93,25 +95,25 @@ sub public {
         my $rsa_err = $_;
 
         try {
-            $key_obj = public_pkcs8($pem_or_der);
+            $key_obj = public_SPKI($pem_or_der);
         }
         catch {
-            die "Failed to parse as either RSA ($rsa_err) or PKCS8 ($_)";
+            die Crypt::Perl::X::create('Generic', "Failed to parse as either RSA ($rsa_err) or SubjectPublicKeyInfo ($_)");
         };
     };
 
     return $key_obj;
 }
 
-#Like public(), but only does PKCS8.
-sub public_pkcs8 {
+#Like public(), but only does SubjectPublicKeyInfo.
+sub public_SPKI {
     my ($pem_or_der) = @_;
 
     _ensure_der($pem_or_der);
 
-    my $pkcs8 = _decode_pkcs8_public($pem_or_der);
+    my $spki = _decode_spki($pem_or_der);
 
-    my $parsed = _decode_rsa_public_within_pkcs8_or_die($pkcs8);
+    my $parsed = _decode_rsa_public_within_spki_or_die($spki);
 
     return _new_public($parsed);
 }
@@ -132,7 +134,7 @@ sub jwk {
 
     my %constr_args;
 
-    Module::Load::load('Crypt::Perl::JWK');
+    require Crypt::Perl::JWK;
 
     for my $k (keys %$hr) {
         next if !$JTK_TO_NEW{$k};
@@ -141,11 +143,11 @@ sub jwk {
 
     if ($hr->{'d'}) {
         $constr_args{'version'} = 0;
-        Module::Load::load('Crypt::Perl::RSA::PrivateKey');
+        require Crypt::Perl::RSA::PrivateKey;
         return Crypt::Perl::RSA::PrivateKey->new( \%constr_args );
     }
 
-    Module::Load::load('Crypt::Perl::RSA::PublicKey');
+    require Crypt::Perl::RSA::PublicKey;
     return Crypt::Perl::RSA::PublicKey->new( \%constr_args );
 }
 
@@ -180,21 +182,21 @@ sub _decode_rsa_within_pkcs8_or_die {
         $dec = _decode_rsa( $pkcs8_hr->{'privateKey'} );
     }
     catch {
-        die "Failed to parse RSA within PKCS8: $_";
+        die Crypt::Perl::X::create('Generic', "Failed to parse RSA within PKCS8: $_");
     };
 
     return $dec;
 }
 
-sub _decode_rsa_public_within_pkcs8_or_die {
-    my ($pkcs8_hr) = @_;
+sub _decode_rsa_public_within_spki_or_die {
+    my ($spki_hr) = @_;
 
     my $dec;
     try {
-        $dec = _decode_rsa_public( $pkcs8_hr->{'subjectPublicKey'}[0] );
+        $dec = _decode_rsa_public( $spki_hr->{'subjectPublicKey'}[0] );
     }
     catch {
-        die "Failed to parse RSA within PKCS8: $_";
+        die Crypt::Perl::X::create('Generic', "Failed to parse RSA within SubjectPublicKeyInfo: $_");
     };
 
     return $dec;
@@ -206,7 +208,7 @@ sub _decode_pkcs8 {
     return _decode_macro( $$der_r, 'PrivateKeyInfo' );
 }
 
-sub _decode_pkcs8_public {
+sub _decode_spki {
     my ($der_r) = (\$_[0]);
 
     return _decode_macro( $$der_r, 'SubjectPublicKeyInfo' );
@@ -215,14 +217,14 @@ sub _decode_pkcs8_public {
 sub _new_public {
     my ($parsed_hr) = @_;
 
-    Module::Load::load('Crypt::Perl::RSA::PublicKey');
+    require Crypt::Perl::RSA::PublicKey;
     return Crypt::Perl::RSA::PublicKey->new($parsed_hr);
 }
 
 sub _new_private {
     my ($parsed_hr) = @_;
 
-    Module::Load::load('Crypt::Perl::RSA::PrivateKey');
+    require Crypt::Perl::RSA::PrivateKey;
     return Crypt::Perl::RSA::PrivateKey->new($parsed_hr);
 }
 
